#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <clapack.h>
#include "util.h"

//----------------------------------------------------------------------
// ASSERT
//      If condition is false,  print a message.
//      Useful for documenting assumptions in the code.
//
//      NOTE: needs to be a #define, to be able to print the location
//      where the error occurred.
//----------------------------------------------------------------------
#define ASSERT(condition)                                                     \
    if (!(condition)) {                                                       \
        fprintf(stderr, "Assertion failed: line %d, file \"%s\"\n",           \
                __LINE__, __FILE__);                                          \
        fflush(stderr);                                                       \
        exit(0);                                                              \
	}

//----------------------------------------------------------------------
// DEBUG
//      Print a debug message, if flag is enabled.  Like printf,
//      only with an extra argument on the front.
//----------------------------------------------------------------------
void DEBUG (char flag, char *format, ...)
{
  if (0 == 0){
      {
          va_list ap;
          // You will get an unused variable message here -- ignore it.
          va_start (ap, format);
          vfprintf (stdout, format, ap);
          va_end (ap);
          fflush (stdout);
      }
  }
}

/*----------------------------------------------------------------------
  C Alloue size octets, et renvoie  un  pointeur  sur  la  memoire
  C allouee.  Le contenu de la zone de memoire n'est pas initialise.
  C Encapsule malloc tout en comptabilisant la memoire allouee.
 */
void * APalloc (size_t size){
  APMemoryAllocated += (int) size;
  return malloc(size);
}
  

long ms_time(void) 
{
  struct timeval tv;

  gettimeofday(&tv, NULL);
  return(tv.tv_sec*1000 + tv.tv_usec/1000);
}


void long_computation(void) 
{
   long now=ms_time();
   while(ms_time()<now+4000) 
     {
     }
}

void printbuffer(double* buffer, int length, int id){
  int i;
  if (length > 100) return;
  printf("%d: [", id);
  for (i = 0 ; i < length - 1 ; i++){
    printf("%e,",buffer[i]);
    }
  printf("%e]\n",buffer[length - 1]);
}

void printbufferofint(int* buffer, int length, int id){
  int i;
  if (length > 100) return;
  printf("%d: [", id);
  for (i = 0 ; i < length - 1 ; i++){
    printf("%d,",buffer[i]);
    }
  printf("%d]\n",buffer[length - 1]);
}


/** Initialise le vecteur v de taille size a la valeur value.
 */
int initarray(double *v, double value, int size){
  int i;
  for (i = 0 ; i < size ; i++){
    v[i] = value;
  }
  return 0;
}

/*-- Calcul de la norme infini d'un vecteur v de taille size*/
double NORM(double *v, int size){
  int i;
  double norm, tmp;
  norm = 0;
  for (i = 0 ; i < size ; i++){
    tmp = v[i];
    if (tmp > norm){
      norm = tmp;
    }
    else if (-tmp > norm){
      norm = - tmp;
    }
  }
  return norm;
}


inline double alea(double Max)
{
    /* d'apres :
     * http://www.eskimo.com/~scs/C-faq/q13.16.html 
     * http://www.ulib.org/webRoot/Books/Numerical_Recipes/bookcpdf.html
     * RAND_MAX est definie dans /usr/include/stdlib.h et vaut (2^15)-1
     */
    return (Max * (double) rand() / ((double) RAND_MAX + 1));
}


/* initialisation du generateur aleatoire */
void randomInit(){
  srand((unsigned int) time(NULL) + getpid());
}


/*-----------------------
  C Genere un vecteur aleatoire de taille N a valeurs comprises
  C entre -Max et Max.
  C Appeler RandomInit() avant tout appel a randomArray.
  C
  C Parametres: 
  C ----------
  C    Output : v
  C    Input  : Max, N
 */
int randomArray(double *v, int N, double Max){
  int indice;
  for (indice = 0 ; indice < N ; indice++){
    v[indice] = alea(2 * Max) - Max;
  }

  return 0;
}

/*-------------------------------------------
  C Generation aleatoire de A et B tels que :
  C - A est une matrice carree d'ordre N
  C a diagonale strictement dominante
  C tridiagonale par blocs composee
  C de Nblocs blocs diagonaux de taille M 
  C (N = Nblocs * M)
  C - B est un vecteur de taille N = Nblocs * M
  C
  C Les valeurs non digonales de A et B seront 
  C comprises entre (-Max) et (+Max), et les 
  C valeurs diagonales de A seront egales a 
  C 3 * M * Max.
  C
  C Parametres: 
  C ----------
  C    Output : A, B
  C    Input  : M, Nblocs, Max
  C------------------------------------------
*/
int generateAB(double *A, double *B, int M, int Nblocs, double Max){

  int blocdiag, bloc, bloconline, blocsperline, row, indice, debut;

  bloc = 0;
  for (blocdiag = 0 ; blocdiag < Nblocs ; blocdiag++){
    if (blocdiag == 0 || blocdiag == Nblocs - 1){
      if (Nblocs == 1){
	blocsperline = 1;
      }
      else{
	blocsperline = 2;
      }
    }
    else{
      blocsperline = 3;
    }
    //printf("blocsperline = %d\n", blocsperline);
    
    /*On parcourt les blocs et on les initialise*/
    for (bloconline = 0; bloconline < blocsperline ; bloconline++){
      debut = bloc * M * M;
      randomArray(A+(debut), M * M, Max);
      /*On rend la matrice a diagonale strictement dominante */
      indice = 0;
      
      if (bloc % 3 == 0){
	//printf("Allo : bloc %d, indice %d, debut %d, ind+deb %d\n", bloc, indice, debut, indice+debut);
	for (row = 0; row < M; row++){
	  A[debut + indice] = 3 * M * Max ;
	  indice += M + 1;
	}
      }
      
      randomArray(B+(blocdiag * M), M, Max);
      bloc++;
    }
  }
  return 0;
}


/* Exemple de resolution d'un systeme lineaire dense D * Y = b 
par factorisation LU grace a la bibiotheque Lapack */
int sampleLU(){
  double *D, *y, *b;
  int *IPIV;
  int M;
  M = 153;
  D = APalloc(M * M * sizeof(double));
  y = APalloc(M * sizeof(double));
  b = APalloc(M * sizeof(double));
  IPIV = malloc(M * sizeof(int));
  ASSERT(D != NULL && y != NULL && b != NULL && IPIV != NULL);
/*      --- Initialiser la matrice et le second membre */
  generateAB(D, b, M, 1, 1.0);
/*      ---------------------------------------------
       factorisation LU de la matrice D (D = P L U)
        ---------------------------------------------*/
  clapack_dgetrf(CblasRowMajor, M, M, D, M, IPIV);
/*         -- En sortie de dgetrf :
C         D(1..M,1..M) contient les matrices L et U
C         IPIV(1..M) contient le vecteur de permutation 
C                    associe a la matrice P
C      ---------------------------------------------
C      Resolution : calcul de y tel que  D y = b
C                              (P L U y = b)
C      ---------------------------------------------*/
  cblas_dcopy(M, b, 1, y, 1);
/*      -- En entree y contient le second membre b*/
  clapack_dgetrs(CblasRowMajor, CblasNoTrans, M, 1, D, M, IPIV, y, M);
/*      -- En sortie y contient la solution de D y = b*/
  free(D);
  free(y);
  free(b);
  free(IPIV);
  return 0;
}
