#ifndef AP_UTIL_H 
#define AP_UTIL_H 

#include <stdlib.h>

/* Comptabilise la memoire allouee avec APalloc().*/
int APMemoryAllocated;

/*--------------------------------------------------------------------
  C Alloue size octets, et renvoie  un  pointeur  sur  la  memoire
  C allouee.  Le contenu de la zone de memoire n'est pas initialise.
  C Encapsule malloc tout en comptabilisant la memoire allouee.
 */
void * APalloc (size_t size);

/*----------------------------------------------------------------------
  C ASSERT
  C  If condition is false,  print a message.
  C     Useful for documenting assumptions in the code.
  C
  C     NOTE: needs to be a #define, to be able to print the location
  C     where the error occurred.
  C----------------------------------------------------------------------*/
#define ASSERT(condition)                                                     \
    if (!(condition)) {                                                       \
        fprintf(stderr, "Assertion failed: line %d, file \"%s\"\n",           \
                __LINE__, __FILE__);                                          \
        fflush(stderr);                                                       \
        exit(0);                                                              \
	}

/*----------------------------------------------------------------------
  C DEBUG
  C   Print a debug message, if flag is enabled.  Like printf,
  C   only with an extra argument on the front.
  C----------------------------------------------------------------------*/
void DEBUG (char flag, char *format, ...);
  
long ms_time(void);

void long_computation(void);

void printbuffer(double* buffer, int length, int id);

void printbufferofint(int* buffer, int length, int id);

/** Initialise le vecteur v de taille size a la valeur value.
 */
int initarray(double *v, double value, int size);

/** Copie les valeurs du vecteur from dans le vecteur to de taille size; 
 */
int copyarray(int *to, int *from, int size);

/*-- Calcul de la norme infini d'un vecteur v de taille size*/
double NORM(double *v, int size);

/* initialisation du generateur aleatoire */
void randomInit();

/*-----------------------
  C Genere un vecteur aleatoire de taille N a valeurs comprises
  C entre -Max et Max.
  C
  C Parametres: 
  C ----------
  C    Output : v
  C    Input  : Max, N
 */
int randomArray(double *v, int N, double Max);

/*-------------------------------------------
  C Generation aleatoire de A et B tels que :
  C - A est une matrice carree d'ordre N
  C a diagonale strictement dominante
  C tridiagonale par blocs composee
  C de Nblocs blocs diagonaux de taille M 
  C (N = Nblocs * M)
  C - B est un vecteur de taille N = Nblocs * M
  C
  C Les valeurs de A et B seront comprises entre
  C (-Max) et (+Max), et les valeurs diagonales
  C de A seront egales a +Max.
  C
  C Parametres: 
  C ----------
  C    Output : A, B
  C    Input  : M, Nblocs, Max
  C------------------------------------------
*/
int generateAB(double *A, double *B, int M, int Nblocs, double Max);
  
/* Exemple de resolution d'un systeme lineaire dense D * Y = b 
par factorisation LU grace a la bibiotheque Lapack */
int sampleLU(void);

#endif
